@@ -1,30 +1,16 @@
-/********************************************************************
-Copyright 2014  Martin Gräßlin <mgraesslin@kde.org>
+/*
+    SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) version 3, or any
-later version accepted by the membership of KDE e.V. (or its
-successor approved by the membership of KDE e.V.), which shall
-act as a proxy defined in Section 6 of version 3 of the license.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 #include "connection_thread.h"
 #include "logging.h"
 // Qt
 #include <QAbstractEventDispatcher>
-#include <QGuiApplication>
 #include <QDebug>
 #include <QDir>
 #include <QFileSystemWatcher>
+#include <QGuiApplication>
 #include <QMutex>
 #include <QMutexLocker>
 #include <QSocketNotifier>
@@ -32,12 +18,12 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 // Wayland
 #include <wayland-client-protocol.h>
 
+#include <poll.h>
+
 namespace KWayland
 {
-
 namespace Client
 {
-
 class Q_DECL_HIDDEN ConnectionThread::Private
 {
 public:
@@ -46,6 +32,7 @@ public:
     void doInitConnection();
     void setupSocketNotifier();
     void setupSocketFileWatcher();
+    void dispatchEvents();
 
     wl_display *display = nullptr;
     int fd = -1;
@@ -57,15 +44,15 @@ public:
     bool foreign = false;
     QMetaObject::Connection eventDispatcherConnection;
     int error = 0;
-    static QVector<ConnectionThread*> connections;
-    static QMutex mutex;
+    static QList<ConnectionThread *> connections;
+    static QRecursiveMutex mutex;
+
 private:
     ConnectionThread *q;
 };
 
-QVector<ConnectionThread*> ConnectionThread::Private::connections = QVector<ConnectionThread*>{};
-QMutex ConnectionThread::Private::mutex{QMutex::Recursive};
-
+QList<ConnectionThread *> ConnectionThread::Private::connections = QList<ConnectionThread *>{};
+QRecursiveMutex ConnectionThread::Private::mutex;
 
 ConnectionThread::Private::Private(ConnectionThread *q)
     : socketName(QString::fromUtf8(qgetenv("WAYLAND_DISPLAY")))
@@ -102,7 +89,7 @@ void ConnectionThread::Private::doInitConnection()
     }
     if (!display) {
         qCWarning(KWAYLAND_CLIENT) << "Failed connecting to Wayland display";
-        emit q->failed();
+        Q_EMIT q->failed();
         return;
     }
     if (fd != -1) {
@@ -114,32 +101,53 @@ void ConnectionThread::Private::doInitConnection()
     // setup socket notifier
     setupSocketNotifier();
     setupSocketFileWatcher();
-    emit q->connected();
+    Q_EMIT q->connected();
 }
 
 void ConnectionThread::Private::setupSocketNotifier()
 {
     const int fd = wl_display_get_fd(display);
     socketNotifier.reset(new QSocketNotifier(fd, QSocketNotifier::Read));
-    QObject::connect(socketNotifier.data(), &QSocketNotifier::activated, q,
-        [this]() {
-            if (!display) {
-                return;
+    QObject::connect(socketNotifier.data(), &QSocketNotifier::activated, q, [this]() {
+        dispatchEvents();
+    });
+}
+
+void ConnectionThread::Private::dispatchEvents()
+{
+    if (!display) {
+        return;
+    }
+    // first dispatch any pending events on the default queue
+    while (wl_display_prepare_read(display) != 0) {
+        wl_display_dispatch_pending(display);
+    }
+    wl_display_flush(display);
+    // then check if there are any new events waiting to be read
+    struct pollfd pfd;
+    pfd.fd = wl_display_get_fd(display);
+    pfd.events = POLLIN;
+    int ret = poll(&pfd, 1, 0);
+    if (ret > 0) {
+        // if yes, read them now
+        wl_display_read_events(display);
+    } else {
+        wl_display_cancel_read(display);
+    }
+
+    // finally, dispatch the default queue and all frame queues
+    if (wl_display_dispatch_pending(display) == -1) {
+        error = wl_display_get_error(display);
+        if (error != 0) {
+            if (display) {
+                free(display);
+                display = nullptr;
             }
-            if (wl_display_dispatch(display) == -1) {
-                error = wl_display_get_error(display);
-                if (error != 0) {
-                    if (display) {
-                        free(display);
-                        display = nullptr;
-                    }
-                    emit q->errorOccurred();
-                    return;
-                }
-            }
-            emit q->eventsRead();
+            Q_EMIT q->errorOccurred();
+            return;
         }
-    );
+    }
+    Q_EMIT q->eventsRead();
 }
 
 void ConnectionThread::Private::setupSocketFileWatcher()
@@ -149,46 +157,45 @@ void ConnectionThread::Private::setupSocketFileWatcher()
     }
     socketWatcher.reset(new QFileSystemWatcher);
     socketWatcher->addPath(runtimeDir.absoluteFilePath(socketName));
-    QObject::connect(socketWatcher.data(), &QFileSystemWatcher::fileChanged, q,
-        [this] (const QString &file) {
-            if (QFile::exists(file) || serverDied) {
+    QObject::connect(socketWatcher.data(), &QFileSystemWatcher::fileChanged, q, [this](const QString &file) {
+        if (QFile::exists(file) || serverDied) {
+            return;
+        }
+        qCWarning(KWAYLAND_CLIENT) << "Connection to server went away";
+        serverDied = true;
+        if (display) {
+            free(display);
+            display = nullptr;
+        }
+        socketNotifier.reset();
+
+        // need a new filesystem watcher
+        socketWatcher.reset(new QFileSystemWatcher);
+        socketWatcher->addPath(runtimeDir.absolutePath());
+        QObject::connect(socketWatcher.data(), &QFileSystemWatcher::directoryChanged, q, [this]() {
+            if (!serverDied) {
                 return;
             }
-            qCWarning(KWAYLAND_CLIENT) << "Connection to server went away";
-            serverDied = true;
-            if (display) {
-                free(display);
-                display = nullptr;
+            if (runtimeDir.exists(socketName)) {
+                qCDebug(KWAYLAND_CLIENT) << "Socket reappeared";
+                socketWatcher.reset();
+                serverDied = false;
+                error = 0;
+                q->initConnection();
             }
-            socketNotifier.reset();
-
-            // need a new filesystem watcher
-            socketWatcher.reset(new QFileSystemWatcher);
-            socketWatcher->addPath(runtimeDir.absolutePath());
-            QObject::connect(socketWatcher.data(), &QFileSystemWatcher::directoryChanged, q,
-                [this]() {
-                    if (!serverDied) {
-                        return;
-                    }
-                    if (runtimeDir.exists(socketName)) {
-                        qCDebug(KWAYLAND_CLIENT) << "Socket reappeared";
-                        socketWatcher.reset();
-                        serverDied = false;
-                        error = 0;
-                        q->initConnection();
-                    }
-                }
-            );
-            emit q->connectionDied();
-        }
-    );
+        });
+        Q_EMIT q->connectionDied();
+    });
 }
 
 ConnectionThread::ConnectionThread(QObject *parent)
     : QObject(parent)
     , d(new Private(this))
 {
-    d->eventDispatcherConnection = connect(QCoreApplication::eventDispatcher(), &QAbstractEventDispatcher::aboutToBlock, this,
+    d->eventDispatcherConnection = connect(
+        QCoreApplication::eventDispatcher(),
+        &QAbstractEventDispatcher::aboutToBlock,
+        this,
         [this] {
             if (d->display) {
                 wl_display_flush(d->display);
@@ -216,7 +223,7 @@ ConnectionThread *ConnectionThread::fromApplication(QObject *parent)
     if (!native) {
         return nullptr;
     }
-    wl_display *display = reinterpret_cast<wl_display*>(native->nativeResourceForIntegration(QByteArrayLiteral("wl_display")));
+    wl_display *display = reinterpret_cast<wl_display *>(native->nativeResourceForIntegration(QByteArrayLiteral("wl_display")));
     if (!display) {
         return nullptr;
     }
@@ -227,7 +234,7 @@ ConnectionThread *ConnectionThread::fromApplication(QObject *parent)
 
 void ConnectionThread::initConnection()
 {
-    QMetaObject::invokeMethod(this, "doInitConnection", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, &ConnectionThread::doInitConnection, Qt::QueuedConnection);
 }
 
 void ConnectionThread::doInitConnection()
@@ -300,10 +307,12 @@ int ConnectionThread::errorCode() const
     return d->error;
 }
 
-QVector<ConnectionThread*> ConnectionThread::connections()
+QList<ConnectionThread *> ConnectionThread::connections()
 {
     return Private::connections;
 }
 
 }
 }
+
+#include "moc_connection_thread.cpp"
